% Copyright (c) 2011 Department of Information Engineering, University of Padova, Italy
% Contributors: Davide Zordan, 7 Mar. 2011
% All rights reserved

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above
%      copyright notice, this list of conditions and the following
%      disclaimer in the documentation and/or other materials provided
%      with the distribution.
%    * The name of the author may not be used to endorse or promote
%      products derived from this software without specific prior
%      written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
% A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
% OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
% SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
% LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
% DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
% THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
% (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
% OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function [synt_data, xc , yc, RoT] = generate_synthetic_signal(dim_grid, dim_time, C,r)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%GENERATE_SYNTHETIC_SIGNAL
% generates a synthetic signal according to the given spatial correlation matrix 
% and temporal correlation coefficient
%
% Inputs:
%   dim_grid                the spatial dimension of the output signal
%   dim_time                the number of time instants to be generated
%   C [dim_gridxdim_grid]   the spatial correlation matrix
%   r                       the temporal correlation coefficient
%
% Outputs:
%   synt_data               signal generated
%    [dim_gridxdim_gridxdim_time]
%   xc, yc                  spatial correlation of the synthetic signal
%   RoT                     temporal correlarion coefficient of the
%                           synthetic signal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% synthetic signal is generated on a square grid 
dim_lato_rete = dim_grid;
% coefficient AR process; time evolution of the loss field 
a_time = r;             
% standard deviation of the noise in the AR process
b_time = sqrt(1 - a_time^2);        
T = dim_time;


R = (1/dim_lato_rete).*fft2(C);
Z_tot = zeros(dim_lato_rete,dim_lato_rete,T);

for i = 1:T  % each time instant 
    
    if i == 1
        Z_iid = randn(dim_lato_rete);
    else
        Z_iid = a_time.*real(Z_iid_t1) + b_time.*randn(dim_lato_rete);
    end
    S_Lf_iid = (1/dim_lato_rete).*(fft2(Z_iid));
    S_Lf = (R.^(1/2)) .*S_Lf_iid;
 
    Z = (dim_lato_rete).*(ifft2(S_Lf));
    Z = real(Z);
    
    Z_iid_t1 = Z_iid;
    
    Z_tot(:,:,i) = Z;
end

% analysis of the synthetic signal
RoT = synt_data_one_step_temporal_correlation(Z_tot);

RoS = [];
for t = 1:T
    [d,ros] = synt_data_spatial_correlation(Z_tot(:,:,t), 20, 50000);
    RoS = [RoS; ros];
end

xc=d./sqrt(2*(99)^2);
yc=mean(RoS);

synt_data = Z_tot;

end
